#!/bin/bash
#Checking if we have NCARG_ROOT variable set and ncl in our path
#============================================================
if [ -z $NCARG_ROOT ]; then
   echo "#####################################"
   echo "YOU NEED TO SET \$NCARG_ROOT VARIABLE"
   echo "PLEASE REFER TO README.ncl"
   echo "#####################################"
 echo "Exiting ..."
 exit
fi

#Checking if we have NCARG_ROOT variable set and ncl in our path
#============================================================
if [ -z $NCLGCCVER ]; then
 echo "#####################################"
 echo "YOU NEED TO SET \$NCLGCCVER VARIABLE"
 echo "PLEASE REFER TO README.ncl"
 echo "#####################################"
 echo "Exiting ..."
 exit
fi

#Compile readfst2d.f readfst3d.f 
#===============================
#In routinesnames you have to provide the name of the fortran routines to be compiled as shared objects

routinesnames=('readfst2d' 'readfst3d')
lengtharray=${#routinesnames[@]}

#a) Generate wrapper C routines

for (( i=0;i<$lengtharray;i++)); do
 
     ${NCARG_ROOT}/bin/wrapit77 <${routinesnames[${i}]}.stub> ${routinesnames[${i}]}_W.c    
done

#b) setting C and Fortran compilers and flags used in generating librmn.a
CCOMPILER="gcc"
FCOMPILER="gfortran"
CCOMPFLAGS="-g -c -m32 -O0"
FCOMPFLAGS="-g -shared -m32 -fcray-pointer"

#c) Check if CCOMPILER and FCOMPILER versions match the ones in env variable NCLGCCVER:
gccversion=$(gcc --version|head -n 1|awk '{print $3}'|awk 'BEGIN{FS="."}{print $1$2$3}')
gfortranversion=$(gfortran --version|head -n 1|awk '{print $4}'|awk 'BEGIN{FS="."}{print $1$2$3}')

if [ ${gccversion} -eq ${NCLGCCVER} ] && [ ${gfortranversion} -eq ${NCLGCCVER} ]  ; then
echo "Creating Shared Objects ..."
echo "Making link to the appropriate librmn.a library ..."
ln -sf ../lib/librmn_gcc${NCLGCCVER}".a" librmn.a 

if [ -e ${ARMNLIB}/librmn.a ] ; then
 echo "==================================================="
 echo "Found the required librmn.a version in " ${ARMNLIB}"
 echo "==================================================="
else
 echo "========================================================"
 echo "You probably have incorrectly set NCLGCCVERSION variable.
 echo "It should be any of 412 421 or 432"
 echo "Exiting ..."
 echo "========================================================"
 exit
fi


#d) Creating shared objects
for (( i=0;i<$lengtharray;i++)); do

$CCOMPILER  -I${NCARG_ROOT}/include ${CCOMPFLAGS}  ${routinesnames[${i}]}_W.c
$FCOMPILER -c ${FCOMPFLAGS}  ${routinesnames[${i}]}.f
$FCOMPILER  ${FCOMPFLAGS}  ${routinesnames[${i}]}.o ${routinesnames[${i}]}_W.o librmn.a -o ${routinesnames[${i}]}.so 

done

#d) Cleaning up and Moving shared objects

rm *.o
rm *_W.c

mv *.so ../shareobjects/.

echo "#########################################################################################"
echo "# Please check that the files .so created by this script have all the libraries resolved#"
echo "# Just type:   ldd   ../shareobjects/filename.so                                        #"
echo "# You will see all the libraries needed by filename.so                                  #"
echo "# In case these libraries are not found, just try to locate them                        #"
echo "# and export the environmental variable LD_LIBRARY_PATH                                 #"
echo "# i.e.   export LD_LIBRARY_PATH="path where the required libraries are"                 #"
echo "#########################################################################################"

else
 echo "============================================================================="
 echo "The gcc/gfortran compiler version found is:${gccversion}/${gfortranversion}  "
 echo "The NCLGCCVER found is                    :${NCLGCCVER}" 
 echo "These two must match"
 echo "See WARNING note in README.ncl"
 echo "============================================================================="
fi
